package com.spring.boot.web.aspect;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

/**
 * 记录REST接口从请求到响应的时间
 * @author yinjihuan
 *
 */
@Component("MethodTimeAspect")
@Aspect
public class MethodTimeAspect {
	private Logger logger = LoggerFactory.getLogger(MethodTimeAspect.class);

    @Around("execution(* com.spring.boot.web.controller.*Controller.*(..))")
    public Object around(ProceedingJoinPoint point) throws Throwable {
        long start = System.currentTimeMillis();
        Object result = null;
        try {
            result = point.proceed();
        } catch (Throwable e) {
            logger.error("", e);
            throw new Throwable(e.getMessage());
        }
        long time = System.currentTimeMillis() - start;
        Class<?> clazz = point.getTarget().getClass();
        String methodName = MethodSignature.class.cast(point.getSignature()).getMethod().getName();
        logger.info("REST API Method:" + clazz.getSimpleName() + "." + methodName + ",Time:" + time + "ms");
        return result;
    }
}
